Imports System.ComponentModel
Imports System.Net.Http.Headers

Namespace GovDataAPI.Areas.HelpPage
	''' <summary>
	''' This is used to identify the place where the sample should be applied.
	''' </summary>
	Public Class HelpPageSampleKey
		''' <summary>
		''' Creates a new <see cref="HelpPageSampleKey"/> based on media type and CLR type.
		''' </summary>
		''' <param name="mediaType">The media type.</param>
		''' <param name="type">The CLR type.</param>
		Public Sub New(ByVal mediaType As MediaTypeHeaderValue, ByVal type As Type)
			If mediaType Is Nothing Then
				Throw New ArgumentNullException("mediaType")
			End If
			If type Is Nothing Then
				Throw New ArgumentNullException("type")
			End If
			ControllerName = String.Empty
			ActionName = String.Empty
			ParameterNames = New HashSet(Of String)(StringComparer.OrdinalIgnoreCase)
			ParameterType = type
			Me.MediaType = mediaType
		End Sub

		''' <summary>
		''' Creates a new <see cref="HelpPageSampleKey"/> based on <see cref="SampleDirection"/>, controller name, action name and parameter names.
		''' </summary>
		''' <param name="sampleDirection">The <see cref="SampleDirection"/>.</param>
		''' <param name="controllerName">Name of the controller.</param>
		''' <param name="actionName">Name of the action.</param>
		''' <param name="parameterNames">The parameter names.</param>
		Public Sub New(ByVal sampleDirection As SampleDirection, ByVal controllerName As String, ByVal actionName As String, ByVal parameterNames As IEnumerable(Of String))
			If Not System.Enum.IsDefined(GetType(SampleDirection), sampleDirection) Then
				Throw New InvalidEnumArgumentException("sampleDirection", CInt(Fix(sampleDirection)), GetType(SampleDirection))
			End If
			If controllerName Is Nothing Then
				Throw New ArgumentNullException("controllerName")
			End If
			If actionName Is Nothing Then
				Throw New ArgumentNullException("actionName")
			End If
			If parameterNames Is Nothing Then
				Throw New ArgumentNullException("parameterNames")
			End If
			Me.ControllerName = controllerName
			Me.ActionName = actionName
			Me.ParameterNames = New HashSet(Of String)(parameterNames, StringComparer.OrdinalIgnoreCase)
			SampleDirection = sampleDirection
		End Sub

		''' <summary>
		''' Creates a new <see cref="HelpPageSampleKey"/> based on media type, <see cref="SampleDirection"/>, controller name, action name and parameter names.
		''' </summary>
		''' <param name="mediaType">The media type.</param>
		''' <param name="sampleDirection">The <see cref="SampleDirection"/>.</param>
		''' <param name="controllerName">Name of the controller.</param>
		''' <param name="actionName">Name of the action.</param>
		''' <param name="parameterNames">The parameter names.</param>
		Public Sub New(ByVal mediaType As MediaTypeHeaderValue, ByVal sampleDirection As SampleDirection, ByVal controllerName As String, ByVal actionName As String, ByVal parameterNames As IEnumerable(Of String))
			If mediaType Is Nothing Then
				Throw New ArgumentNullException("mediaType")
			End If
			If Not System.Enum.IsDefined(GetType(SampleDirection), sampleDirection) Then
				Throw New InvalidEnumArgumentException("sampleDirection", CInt(Fix(sampleDirection)), GetType(SampleDirection))
			End If
			If controllerName Is Nothing Then
				Throw New ArgumentNullException("controllerName")
			End If
			If actionName Is Nothing Then
				Throw New ArgumentNullException("actionName")
			End If
			If parameterNames Is Nothing Then
				Throw New ArgumentNullException("parameterNames")
			End If
			Me.ControllerName = controllerName
			Me.ActionName = actionName
			Me.MediaType = mediaType
			Me.ParameterNames = New HashSet(Of String)(parameterNames, StringComparer.OrdinalIgnoreCase)
			SampleDirection = sampleDirection
		End Sub

		''' <summary>
		''' Gets the name of the controller.
		''' </summary>
		''' <value>
		''' The name of the controller.
		''' </value>
		Private privateControllerName As String
		Public Property ControllerName() As String
			Get
				Return privateControllerName
			End Get
			Private Set(ByVal value As String)
				privateControllerName = value
			End Set
		End Property

		''' <summary>
		''' Gets the name of the action.
		''' </summary>
		''' <value>
		''' The name of the action.
		''' </value>
		Private privateActionName As String
		Public Property ActionName() As String
			Get
				Return privateActionName
			End Get
			Private Set(ByVal value As String)
				privateActionName = value
			End Set
		End Property

		''' <summary>
		''' Gets the media type.
		''' </summary>
		''' <value>
		''' The media type.
		''' </value>
		Private privateMediaType As MediaTypeHeaderValue
		Public Property MediaType() As MediaTypeHeaderValue
			Get
				Return privateMediaType
			End Get
			Private Set(ByVal value As MediaTypeHeaderValue)
				privateMediaType = value
			End Set
		End Property

		''' <summary>
		''' Gets the parameter names.
		''' </summary>
		Private privateParameterNames As HashSet(Of String)
		Public Property ParameterNames() As HashSet(Of String)
			Get
				Return privateParameterNames
			End Get
			Private Set(ByVal value As HashSet(Of String))
				privateParameterNames = value
			End Set
		End Property

		Private privateParameterType As Type
		Public Property ParameterType() As Type
			Get
				Return privateParameterType
			End Get
			Private Set(ByVal value As Type)
				privateParameterType = value
			End Set
		End Property

		''' <summary>
		''' Gets the <see cref="SampleDirection"/>.
		''' </summary>
		Private privateSampleDirection? As SampleDirection
		Public Property SampleDirection() As SampleDirection?
			Get
				Return privateSampleDirection
			End Get
			Private Set(ByVal value? As SampleDirection)
				privateSampleDirection = value
			End Set
		End Property

		Public Overrides Overloads Function Equals(ByVal obj As Object) As Boolean
			Dim otherKey As HelpPageSampleKey = TryCast(obj, HelpPageSampleKey)
			If otherKey Is Nothing Then
				Return False
			End If

			Return String.Equals(ControllerName, otherKey.ControllerName, StringComparison.OrdinalIgnoreCase) AndAlso String.Equals(ActionName, otherKey.ActionName, StringComparison.OrdinalIgnoreCase) AndAlso (MediaType Is otherKey.MediaType OrElse (MediaType IsNot Nothing AndAlso MediaType.Equals(otherKey.MediaType))) AndAlso ParameterType = otherKey.ParameterType AndAlso SampleDirection = otherKey.SampleDirection AndAlso ParameterNames.SetEquals(otherKey.ParameterNames)
		End Function

		Public Overrides Function GetHashCode() As Integer
			Dim hashCode As Integer = ControllerName.ToUpperInvariant().GetHashCode() Xor ActionName.ToUpperInvariant().GetHashCode()
			If MediaType IsNot Nothing Then
				hashCode = hashCode Xor MediaType.GetHashCode()
			End If
			If SampleDirection IsNot Nothing Then
				hashCode = hashCode Xor SampleDirection.GetHashCode()
			End If
			If ParameterType IsNot Nothing Then
				hashCode = hashCode Xor ParameterType.GetHashCode()
			End If
			For Each parameterName As String In ParameterNames
				hashCode = hashCode Xor parameterName.ToUpperInvariant().GetHashCode()
			Next parameterName

			Return hashCode
		End Function
	End Class
End Namespace
