﻿Imports System.Data.Entity
Imports System.Data.Entity.Infrastructure
Imports System.Net
Imports System.Net.Http
Imports System.Web.Http
Imports GovDataAPI.Models
Imports System.Data.Spatial

Namespace GovDataAPI.Controllers
	Public Class ItemsController
		Inherits ApiController
		Private db As New GovDataStoreEntities()

		' GET api/Items
		Public Function GetItems() As IEnumerable(Of Item)
			Dim items = db.Items.Include(Function(i) i.Source)
			Return items.AsEnumerable()
		End Function

		' GET api/Items/5
		Public Function GetItem(ByVal id As Integer) As Item
			Dim item As Item = db.Items.Find(id)
			If item Is Nothing Then
				Throw New HttpResponseException(Request.CreateResponse(HttpStatusCode.NotFound))
			End If

			Return item
		End Function

		''' <summary>
		''' Returns content from the items table within the specified radius
		''' </summary>
		''' <param name="latitude">Reference Latitude</param>
		''' <param name="longitude">Reference Longitude</param>
		''' <param name="radius">Search Radius in miles</param>
		''' <param name="text">Optional: Item text</param>
		''' <returns></returns>
		Public Function GetItems(ByVal latitude As Double, ByVal longitude As Double, ByVal radius As Double, Optional ByVal text As String = "") As IEnumerable(Of SearchResult)
			Dim point As DbGeography = DbGeography.PointFromText(String.Format("POINT({0} {1})", longitude, latitude), DbGeography.DefaultCoordinateSystemId)
			Dim distance As Double = MilesToMeters(radius)
			Dim results As New List(Of SearchResult)()

			db.Items.Where(Function(i) (i.Title.Contains(text) OrElse i.Description.Contains(text) OrElse String.IsNullOrEmpty(text)) AndAlso i.Location.Distance(point) < distance).ToList().ForEach(Sub(r) results.Add(New SearchResult(r)))

			Return results.AsEnumerable()
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			db.Dispose()
			MyBase.Dispose(disposing)
		End Sub

		''' ref: Basic Spatial Data with SQL Server and Entity Framework 5.0 - by Rick Strahl  - http://bit.ly/MuaLUS 
		''' 
		''' Convert meters to miles
		''' 
		''' 
		''' 
		Public Shared Function MetersToMiles(ByVal meters? As Double) As Double
			If meters Is Nothing Then
				Return 0F
			End If

			Return meters.Value * 0.000621371192
		End Function

		''' 
		''' Convert miles to meters
		''' 
		''' 
		''' 
		Public Shared Function MilesToMeters(ByVal miles? As Double) As Double
			If miles Is Nothing Then
				Return 0
			End If

			Return miles.Value * 1609.344
		End Function
	End Class
End Namespace