﻿Imports GovDataVisApp.Commands
Imports GovDataVisApp.Models
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.IO
Imports System.Linq
Imports System.Net
Imports System.Net.Http
Imports System.Net.Http.Headers
Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports System.Windows.Input
Imports System.Xml.Linq

Namespace GovDataVisApp.ViewModels
	Public Class MapSearchViewModel
		Implements INotifyPropertyChanged


		Public Sub New()
			Me.SearchResults = New ObservableCollection(Of MapSearchResult)()
			Me.IsWorking = False

			' set up the search results load command
			Me._loadSearchResults = New DelegateCommand(AddressOf Me.LoadGeoSearchApiResults)

			Me._query = New MapQuery()
		End Sub

		Private _baseAddressUrl As String
		Public Property BaseAddressUrl() As String
			Get
				Return Me._baseAddressUrl
			End Get

			Set(ByVal value As String)
				Me._baseAddressUrl = value
				Me.NotifyPropertyChanged()
			End Set
		End Property

		Private _isWorking As Boolean = False
		Public Property IsWorking() As Boolean
			Get
				Return Me._isWorking
			End Get
			Set(ByVal value As Boolean)
				Me._isWorking = value
				Me.NotifyPropertyChanged()
			End Set
		End Property

		Private _queryComplete As Boolean = False
		Public Property QueryComplete() As Boolean
			Get
				Return Me._queryComplete
			End Get
			Set(ByVal value As Boolean)
				Me._queryComplete = value
				Me.NotifyPropertyChanged()
			End Set
		End Property

		Private _loadSearchResults As ICommand
		Public ReadOnly Property LoadSearchResults() As ICommand
			Get
				Return Me._loadSearchResults
			End Get
		End Property

		Private _query As MapQuery
		Public Property Query() As MapQuery
			Get
				Return Me._query
			End Get
			Set(ByVal value As MapQuery)
				Me._query = value
				Me.NotifyPropertyChanged()
			End Set
		End Property

		' pulled from the msdn wp8 twitter sample: http://code.msdn.microsoft.com/wpapps/TwitterSearch-Windows-b7fc4e5e
		Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
		Private Sub NotifyPropertyChanged(Optional <CallerMemberName> ByVal propertyName As String = "")
			Dim handler As PropertyChangedEventHandler = PropertyChanged
			If Nothing IsNot handler Then
				handler(Me, New PropertyChangedEventArgs(propertyName))
			End If
		End Sub

		Private _selectedResult As MapSearchResult = Nothing
		Public Property SelectedResult() As MapSearchResult
			Get
				Return Me._selectedResult
			End Get
			Set(ByVal value As MapSearchResult)
				Me._selectedResult = value
				Me.NotifyPropertyChanged()
			End Set
		End Property

		Private async Sub LoadGeoSearchApiResults(ByVal p As Object)
			Try
				' load the results and populate the collection
				Me.SearchResults.Clear()

				Me.IsWorking = True
				Me.QueryComplete = False

				Dim client As New HttpClient()
				client.BaseAddress = New Uri(Me.BaseAddressUrl) ' default: http://govdata.azurewebsites.net/

				client.DefaultRequestHeaders.Accept.Add(New MediaTypeWithQualityHeaderValue("text/xml"))

				Dim response As HttpResponseMessage = await client.GetAsync(String.Format("/api/Items?text={0}&latitude={1}&longitude={2}&radius={3}", New Object() { Me.Query.Text, Me.Query.Latitude, Me.Query.Longitude, Me.Query.Radius }))

				Using stream As Stream = await response.Content.ReadAsStreamAsync()
					Dim doc As XDocument = XDocument.Load(stream)
					Dim defaultNs As XNamespace = doc.Root.GetDefaultNamespace()
					Dim results = doc.Descendants(defaultNs + "SearchResult")

					For Each result In results
						Me.SearchResults.Add(New MapSearchResult() With {.Title = result.Descendants(defaultNs + "Title").First().Value, .Description = result.Descendants(defaultNs + "Description").First().Value, .Latitude = Convert.ToDouble(result.Descendants(defaultNs + "Latitude").First().Value), .Longitude = Convert.ToDouble(result.Descendants(defaultNs + "Longitude").First().Value)})
					Next result
				End Using
			Catch ex As Exception
				Throw ex
			Finally
				Me.QueryComplete = True
				Me.IsWorking = False
			End Try
		End Sub

		Private privateSearchResults As ObservableCollection(Of MapSearchResult)
		Public Property SearchResults() As ObservableCollection(Of MapSearchResult)
			Get
				Return privateSearchResults
			End Get
			Private Set(ByVal value As ObservableCollection(Of MapSearchResult))
				privateSearchResults = value
			End Set
		End Property
	End Class
End Namespace
